import java.util.*;

//********************************************************************
//  PriorityQueue.java      Author: Mark Allen Wiess/Ryo (Rio) Akasaka
//  
//  Last modified: Mon April 2, 2007
//
//  Description: Uses binary heap method to create a Priority Queue with
//  constructor, add, remove, size, and element methods necessary
//  to run the Huffman file compression algorithm. References from
//  Mark Allen Weiss' algorithm.
//********************************************************************

public class PriorityQueue<AnyType> 
{
  private static final int LIMIT = 100;
  private int size;   // Number of elements in heap
  private AnyType [] array; // The heap array
  
  /**
   * Construct an empty PriorityQueue.
   */
  public PriorityQueue()
  {
    size = 0;
    array = (AnyType[]) new Object[LIMIT + 1];
    System.out.println("Using leo's Binary Heap priority queue.");
  }
  
  /**
   * Compares left and right using comparator.
   */
  private int compare(AnyType left, AnyType right)
  {
    return ((Comparable) left).compareTo(right);
  }
  
  /**
   * Add method. Compares object to add
   * and values in binary heap and switches
   * position if child is less than parent
   * @param AnyType x
   * @return true
   */
  public boolean add(AnyType x)
  {
    if(size + 1 == array.length)
      doubleArray(); // Double the length of the array
    
    // Percolate up
    int hole = ++size;
    array[0] = x;
    
    for(; compare(x, array[hole/2]) < 0; hole/=2)
      array[hole] = array[hole/2];
    array[hole] = x;
        
    return true;
    }
  
  /**
   * Size method. 
     * @return the number of items in Priority Queue.
     */  
  public int size()
  {
    return size;
  }
  
  /**
   * Returns the smallest item in the priority queue.
   * @return the smallest item.
   * @throws NoSuchElementException if empty.
   */
  public AnyType element()
  {
    if(size == 0)
      throw new NoSuchElementException();
    return array[1];
  }
  
  /**
   * Removes first (smallest) element
   * and returns it.
   * @return the smallest item.
   * @throws NoSuchElementException if empty.
   */
  public AnyType remove()
  {
    if(size == 0)
      throw new NoSuchElementException();
    AnyType minItem = element();
    array[1] = array[size--];
    percolateDown(1);
    return minItem;
  }
  
  /**
   * Internal method to percolate down in the heap.
   * @param hole the index at which the percolate begins.
   */
  private void percolateDown(int hole)
  {
    int child;
    AnyType temp = array[hole];
    
    for(; hole*2 <= size; hole=child)
      {
	child = hole * 2;
	if(child != size &&
	   compare(array[child + 1], array[child]) < 0)
	  child++;
	if(compare( array[child], temp) < 0)
	  array[hole] = array[child];
	else
	  break;
      }
    array[hole] = temp;
  }
  
  /**
   * Internal method to double the length of the array
   * if size increases.
   */
  private void doubleArray()
  {
    AnyType [] newArray;
    
    newArray = (AnyType []) new Object[array.length * 2];
    for( int i = 0; i < array.length; i++)
      newArray[i] = array[i]; // Copy everything into new array
    array = newArray; // Rename
  }
}